<?php
namespace Softneta\MedDream\Core;

define("STRING_FILTER", FILTER_SANITIZE_STRING | FILTER_SANITIZE_SPECIAL_CHARS);

class HttpUtils
{
    private static $ERROR_MESSAGES = array(
        401 => 'Unauthorized',
        403 => 'Forbidden',
        404 => 'Not Found',
        500 => 'Internal Server Error'
    );

    public static function isEmpty($data) {
        if (is_numeric($data)) {
            return false;
        }

        return empty($data);
    }

    public static function getValueHtmlStrict($data, $key)
    {
        $result = self::getValueHtml($data, $key, '');
        if (self::isEmpty($result)) {
            self::error('missing ' . $key);
        }
        return $result;
    }

    public static function getValueHtml($data, $key, $default)
    {
        if (!array_key_exists($key, $data)) {
            return $default;
        }
        $result = trim($data[$key]);
        if (self::isEmpty($result)) {
            return $default;
        }
        // escape javascript and some tags
        return preg_replace(array(
                '/<(a|script)[^>]*>/i', 
                '/(on\w+)(\s+|)=(\s+|)/i', 
                '/alert\((.*)\)/i',
                '/<script(.*)>/i'
            ), '', $result);
    }

    public static function getValueStrict($data, $key, $filter = STRING_FILTER)
    {
        $result = self::getValue($data, $key, '', $filter);
        if (self::isEmpty($result)) {
            self::error('missing ' . $key);
        }
        return $result;
    }

    public static function getValue($data, $key, $default = null, $filter = STRING_FILTER)
    {
        if (!array_key_exists($key, $data)) {
            return $default;
        }
        return self::getSanitize($data[$key], $default, $filter);
    }

    public static function getSanitize($result, $default = null, $filter = STRING_FILTER)
    {
        if (!is_string($result)) {
            return $result;
        }
        $result = trim($result);
        if (empty($result)) {
            return $default;
        }
        if ($filter != null) {
            $data = filter_var($result, $filter);
            if ($data === false) {
                return $default;
            }
            $result = $data;
        }
        return $result;
    }

    public static function getSanitizeObject($data, $filter = STRING_FILTER)
    {
        if (empty($data)) {
            return $data;
        }
        if (is_array($data)) {
            foreach ($data as $key => $value) {
                $data[$key] = self::getSanitizeObject($value, $filter);
            }
        }  else {
            if (!is_numeric($data)) {
                $data = filter_var($data, $filter);
            }
        }
        return $data;
    }

    public static function getParam($var, $default = null)
    {
        if (isset($_REQUEST[$var]))
            return $_REQUEST[$var];
        if ($default === null)
            self::error('missing ' . $var);
        return $default;
    }

    public static function errorFromArray($error)
    {
        self::error(array('code' => $error[1], 'detail' => $error[2]));
    }

    public static function error($error, $code = 500)
    {
        if (function_exists('http_response_code'))
            http_response_code($code);
        elseif (isset(self::$ERROR_MESSAGES[$code])) {
            $protocol = (isset($_SERVER['SERVER_PROTOCOL']) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0');
            header($protocol . ' ' . $code . ' ' . self::$ERROR_MESSAGES[$code]);
        }
        self::returnJSON(array('error' => $error));
    }

    public static function returnJSON($value)
    {
        header('Content-Type: application/json');
        exit(json_encode($value));
    }
}
