<?php
/*
	Original name: saveImage.php

	Copyright: Softneta, 2017

	Classification: public

	Owner: tb <tomas.burba@softneta.com>

	Contributors:
		td <tomas2mb@gmail.com>
		tb <tomas.burba@softneta.com>
		kf <kestutis.freigofas@softneta.com>

	Description:
		Support for the Save Images dialog in the single image mode
 */

use Softneta\MedDream\Core\ActionFilter;
use Softneta\MedDream\Core\RetrieveStudy;
use Softneta\MedDream\Core\Study;
use Softneta\MedDream\Core\PathUtils;
use Softneta\MedDream\Core\Backend;
use Softneta\MedDream\Core\Audit;
use Softneta\MedDream\Core\Logging;

function updateType(&$type)
{
	//remove dot
	$type = str_replace('.', '', $type);
	if ($type == '.jpg')
		$type = 'jpeg';
}

function getTimestamp()
{
	list($usec, $sec) = explode(" ", microtime());
	list($int, $dec) = explode(".", $usec);
	$dec = date("YmdHis").sprintf("%06d", (int)($dec / 100));
	return $dec;
}

function setVideo($clientid,$filename,$path,$type,$speed=64)
{
	global $log;
	$tmp = __DIR__.DIRECTORY_SEPARATOR.'temp'.DIRECTORY_SEPARATOR;
	$headerFile = meddream_convert2($clientid, dirname(__FILE__), $path, $tmp, 0);
	if (is_array($headerFile))
		$headerFile = $headerFile['result'];

	if (file_exists($headerFile))
	{
		$matches = array();
		preg_match("/<pixelOffset>(.*?)<\/pixelOffset><pixelSize>(.*?)<\/pixelSize>/", file_get_contents($headerFile), $matches);
		@unlink($headerFile);

		$pixelOffset = $matches[1];
		$size = $matches[2];

		$log->asDump("mp4 tmp: '$path', $pixelOffset, $size");

		header("Content-length: " . $size);
		header("Cache-Control: cache, must-revalidate");
		header("Pragma: public");
		header("Content-disposition: file; filename=$filename");
		header('Content-Type: video/'.$type);

		$fp = fopen($path, "rb");
		fseek($fp, $pixelOffset);
		$readSize = round($speed*1024);

		while (!feof($fp) && !connection_aborted() )
		{
			echo fread($fp, $readSize);
			flush();
			if (SLOW_DOWN)
				usleep(SLOW_DOWN);
		}
		fclose($fp);
	}
}

function saveJPG($path, $xfersyntax, $bitsstored, $filename, $type='.jpg')
{
	global $authDB, $smooth, $log;

	if ($xfersyntax == '')
	{
		$xfersyntax = meddream_extract_transfer_syntax(__DIR__, $path);
		if (!strlen($xfersyntax) || ($xfersyntax[0] == '*'))
		{
			$log->asErr('meddream_extract_transfer_syntax: ' . var_export($xfersyntax, true));
			$xfersyntax = '';
		}
	}
	if ($bitsstored == '')
		$bitsstored = 8;		/* irrelevant for a long time, probably since v3 */

	$thumbnail = str_replace("\\", "/", dirname(__FILE__)) . "/temp/image-" . getTimestamp() . ".tmp";
	$basedir = __DIR__;
	$thumbnailSize = 10000;
	$log->asDump('meddream_thumbnail(', $path, ', ', $thumbnail, ', ', $basedir, ', ', $thumbnailSize,
		', ', $xfersyntax, ', ', $bitsstored, ', 0, 0, ', $smooth, ')');
	$r = meddream_thumbnail($path, $thumbnail, $basedir, $thumbnailSize, $xfersyntax,
		$bitsstored, 0, 0, $smooth);
	$log->asDump("meddream_thumbnail: ",substr($r, 0, 6));

	if (strlen($r) < 1) return "";

	if ($r[0] == "E")
	{
		$r = substr($r, 5);

		if (file_exists($r))
			copy($r, $thumbnail);
	}
	elseif ($r[0] == '2')
	{
		$r = substr($r, 5);
		$r = imagecreatefromstring($r);

		if ($type == '.tiff')
		{
			imagepng($r, $thumbnail);

			//convert to tiff
			$newfile = convertImage::convertToType($thumbnail, $type);
			if ($newfile == '')
			{
				if (convertImage::$error != '')
				{
					$log->asErr('Conversion error:'.convertImage::$error);
					$log->asDump('Conversion output:',convertImage::$out);
				}
				else
					imagejpeg($r, $thumbnail, 80);
			}
			else
			{
				@unlink($thumbnail);
				$thumbnail = $newfile;
			}
		}
		else
			imagejpeg($r, $thumbnail, 80);

		imagedestroy($r);
	}
	else
		exit('processing failed, see logs');

	$size = filesize($thumbnail);

	header('Pragma: public');
	header('Cache-Control: cache, must-revalidate');
	header("Content-Length: $size");
	header('Content-Type: image/'.updateType($type));
	header('Content-disposition: attachment;filename="'.$filename.'"');
	echo file_get_contents($thumbnail);
	flush();

	@unlink($thumbnail);
}

if (!strlen(session_id()))
	@session_start();

require_once(__DIR__ . '/autoload.php');
require_once('convertImage.php');

define('SLOW_DOWN', 0);			/* for SWS: 10000 */
set_time_limit(0);
ini_set('memory_limit', '1024M');

$modulename = basename(__FILE__);

$log = new Logging();
$log->asDump('begin ' . $modulename);

$audit = new Audit('SAVE IMAGE');	/* after session_start() for proper session ID tracking */

if (isset($_GET['uid']))
	$uid = urldecode($_GET['uid']);
if (!strlen($uid))
{
	$audit->log(false);
	$log->asWarn('nothing to do');
	exit;
}

$flt = new ActionFilter($log);
$flt->getActionsInto($actions);
$err = $flt->validateAtImageLevel($uid, $actions);
if ($err)
	exit($err);

$clientid = '';
if (isset($_SESSION['clientIdMD']))
	$clientid = $_SESSION['clientIdMD'];

$backend = new Backend(array('Structure', 'Preload'));
if (!$backend->authDB->isAuthenticated() || ($clientid == ''))
{
	$audit->log(false, $uid);
	$log->asErr('not authenticated');
	return false;
}

$type = '.jpg';
if (isset($_GET['type']))
	$type = urldecode($_GET['type']);

$smooth = $backend->enableSmoothing;
if (isset($_REQUEST['smooth']))
	$smooth = intval(urldecode($_REQUEST['smooth']));

$log->asDump('$uid = ', $uid, ', $type = ', $type, ', $smooth = ', $smooth);

if ($backend->pacsConfig->getRetrieveEntireStudy())
{
	$retrieve = new RetrieveStudy(new Study(), $log);
	$err = $retrieve->verifyAndFetch($uid);
	if ($err)
	{
		$audit->log(false, $uid);
		exit($err);
	}
}
$st = $backend->pacsStructure->instanceGetMetadata($uid, true);
if (strlen($st['error']))
{
	$audit->log(false, $uid);
	exit($st['error']);
}
$path = $st['path'];
$xfersyntax = $st['xfersyntax'];
$bitsstored = $st['bitsstored'];
$name = PathUtils::getName($st);

$convert = false;
$type = strtolower($type);
$speed = 64;

switch (strtolower($xfersyntax))
{
	case '1.2.840.10008.1.2.4.100':
		if ($type != '.dcm')
		{
			$convert = true;
			$type = '.mpg';
		}
		break;

	case '1.2.840.10008.1.2.4.103':
		if ($type != '.dcm')
		{
			$convert = true;
			$type = '.mp4';
		}
		break;

	case 'mp4':
		$convert = false;
		$type = '.mp4';
		break;

	default:
		if ($type != '.dcm')
			$convert = true;
		else
			$convert = false;
		break;
}

$name = PathUtils::escapeFileName($name);
$filename = ($name ?: 'Exported') . date('Ymd') . $type;

$log->asDump('$convert = ', $convert, ', $path = ', $path, ', $filename = ', $filename);
if (!$convert)
{
	$size = @filesize($path);
	$fp = @fopen($path, "rb");
	if (!$fp)
	{
		$audit->log(false, $uid);
		$log->asErr('unreadable: ' . var_export($path, true));
		exit('Failed to read the source file');
	}

	header('Pragma: public');
	header('Cache-Control: cache, must-revalidate');
	header("Content-Length: $size");
	header('Content-Type: application/octet-stream');
	header('Content-Disposition: attachment; filename="' . $filename . '"');

	$readSize = round($speed*1024);

	while (!feof($fp) && !connection_aborted())
	{
		echo fread($fp, $readSize);
		flush();
		if (SLOW_DOWN)
			usleep(SLOW_DOWN);
	}
	fclose($fp);
}
else
{
	if (($type == '.mp4') || ($type == '.mpg'))
		setVideo($clientid, $filename, $path, $type, $speed);
	else
		if (($type == '.jpg') || ($type == '.tiff'))
			saveJPG($path, $xfersyntax, $bitsstored, $filename, $type);
}

/* also remove the source file that might be created by instanceGetMetadata() */
$backend->pacsPreload->removeFetchedFile($path);

$audit->log(true, $uid);

$log->asDump('end ' . $modulename);
