<?php
/*
	Original name: getThumbnail.php

	Copyright: Softneta, 2018

	Classification: public

	Description:
		The "Get Thumbnail" service.
		Move this file to MedDream installation directory in order to use.
 */
session_start();
include __DIR__ . '/autoload.php';

use Softneta\MedDream\Core\Study;
use Softneta\MedDream\Core\RetrieveStudy;
use Softneta\MedDream\Core\Backend;
use Softneta\MedDream\Core\Logging;
use Softneta\MedDream\Core\Audit;


$modulename = basename(__FILE__);
$log = new Logging();
$log->asDump('begin ' . $modulename);

$audit = new Audit('GET THUMBNAIL');


/* set error header and stop the script */
function exitWithError()
{
	header("HTTP/1.0 404 Not Found");
	exit;
}


/* create thumbnail and return path to temporary file
 *
 * @global Backend $backend
 * @global Logging $log
 * @param string $uid
 * @param int $thumbnailSize
 * @return string
 */
function getJpg($uid, $thumbnailSize)
{
	global $backend;
	global $log;

	if ($backend->pacsConfig->getRetrieveEntireStudy())
	{
		$retrieve = new RetrieveStudy(new Study(), $log);
		$err = $retrieve->verifyAndFetch($uid);
		if ($err)
			return '';
	}
	$meta = $backend->pacsStructure->instanceGetMetadata($uid);
	if ($meta['error'])
		return '';
	$path = $meta['path'];
	$xfersyntax = $meta['xfersyntax'];
	$bitsstored = $meta['bitsstored'];
	$smooth = $backend->enableSmoothing;
	$uidClean = $meta['uid'];

	$thumbnail = __DIR__ . '/temp/' .$uidClean.'.thumbnail-'.$thumbnailSize.".jpg";

	$log->asDump("meddream_thumbnail('$path', '$thumbnail', $thumbnailSize, '$xfersyntax', $bitsstored)");
	$r = meddream_thumbnail($path, $thumbnail, __DIR__, $thumbnailSize, $xfersyntax,
		$bitsstored, 0, 0, $smooth);
	$log->asDump('meddream_thumbnail: ', substr($r, 0, 6));

	if (strlen($r) > 0)
	{
		if ($r[0] == "E")			/* path to an already existing thumbnail */
		{
			$r = substr($r, 5);
			if (file_exists($r))
				$thumbnail = $r;
		}
		else
			if ($r[0] == "2")		/* GD2 data */
			{
				$r = substr($r, 5);
				$r = imagecreatefromstring($r);
				ob_start();
				imagejpeg($r, NULL, 90);
				$buf = ob_get_contents();
				file_put_contents($thumbnail, $buf);
				imagedestroy($r);
				ob_end_clean();

			}
		/* otherwise return as is; for example, it could be '?PDF' -- indication
		   to display a PDF icon
		 */
	}
	else
	{
		$log->asErr("meddream_thumbnail failed on '$path'");
		$thumbnail = '';
	}

	/* also remove the source file that might be created by instanceGetMetadata() */
	$backend->pacsPreload->removeFetchedFile($path);

	$log->asDump('return: ', $thumbnail);
	return $thumbnail;
}


/* validate parameters */
$log->asDump('$_REQUEST = ', $_REQUEST);
if (!isset($_REQUEST['image']) || empty($_REQUEST['image']))
{
	$log->asErr('missing parameter(s)');
	$audit->log(false);
	exitWithError();
}
$imageUid = $_REQUEST['image'];
$size = 50;
if (isset($_REQUEST['size']) && !empty($_REQUEST['size']))
{
	$size = (int) $_REQUEST['size'];
	$size = min($size, 4320);
	$size = max($size, 50);
}
$log->asDump('$imageUid = ', $imageUid, ', $size = ', $size);

/* log in */
$backend = new Backend(array('Structure', 'Preload'));
$authDB = $backend->authDB;
$alreadyLoggedIn = $authDB->isAuthenticated();
if ($alreadyLoggedIn)
	$log->asDump('reusing and keeping an existing login');
else
{
	if (!file_exists(__DIR__ . '/external.php'))
	{
		$audit->log(false, $imageUid);
		$log->asErr("Can't login: missing or bad external.php");
		exitWithError();
	}
	include __DIR__ . '/external.php';
	if (!$authDB->login(SHOW_DB, SHOW_USER, SHOW_PASSWORD))
	{
		$audit->log(false, $imageUid);
		$authDB->logoff();
		$log->asErr("Can't login: missing or bad external.php");
		exitWithError();
	}
}
$log->asDump('Auth: ', $authDB->isAuthenticated());

/* generate a thumbnail */
$thumbnailPath = getJpg($imageUid, $size);
if (!$alreadyLoggedIn)
	$authDB->logoff();
if (!strlen($thumbnailPath) || !@file_exists($thumbnailPath))
{
	$audit->log(false, $imageUid);
	exitWithError();
}

/**
 * Do not change header "Expires"
 *
 * Firefox Bug 583351
 * https://bugzilla.mozilla.org/show_bug.cgi?id=583351
 */
header('Pragma: public');
header('Expires: ' . date('D, d M Y H:i:s GMT', strtotime('+1 seconds')));
header('Cache-Control: maxage=0');
header('Content-Type: image/jpeg');
echo file_get_contents($thumbnailPath);
@unlink($thumbnailPath);
$audit->log(true, $imageUid);

$log->asDump('end ' . $modulename);
